subroutine euler_matrix(       &
                        a1,    &
                        a2,    &
                        a3,    &
                        matrix &
                                )

   ! --------------------------------------------------------------
   !
   ! this subroutine computes the Euler matrix corresponding
   ! to aeronautics conventionn following the prescription
   ! 	rotation a1 around original Z
   !    rotation a2 around interm   Y
   !    rotation a3 around final    X
   ! see H. Goldestein, Classical Mechanics (2nd Ed.) p. 147 for 
   ! discussion
   ! INPUTS:
   !        a1, a2, a3 = Euler angles, in radians
   !	    all the angles are measured counterclockwise
   !
   ! MODIFICATION HISTORY:
   !    October 1997, Eric Hivon, TAC (original IDL code)
   !    January 1999, A.J. Banday MPA (conversion to F90)
   !    March 22 1999, E.H. Caltech   (modification to match IDL version,
   !    correction of a bug on the matrix product)
   !
   ! --------------------------------------------------------------

   USE healpix_types
   
   IMPLICIT NONE

   REAL(DP)                 :: a1,a2,a3, c1,c2,c3, s1,s2,s3
   REAL(DP)                 :: ze,un
   REAL(DP), DIMENSION(3,3) :: matrix
   REAL(DP), DIMENSION(3,3) :: m1,m2,m3

   ! --------------------------------------------------------------

   !-- convert to sin and cosine values --
   c1 = COS(a1)
   s1 = SIN(a1)
   c2 = COS(a2)
   s2 = SIN(a2)
   c3 = COS(a3)
   s3 = SIN(a3)

   ze = 0.0_dp
   un = 1.0_dp

   !-- form full rotation matrix --
   !-- FOR REFERENCE:
   ! m1 = [[ c1, s1,  0],[-s1, c1,  0],[  0,  0,  1]] ! around orig  z
   ! m2 = [[ c2,  0, s2],[  0,  1,  0],[-s2,  0, c2]] ! around orig  y
   ! m3 = [[  1,  0,  0],[  0, c3, s3],[  0,-s3, c3]] ! around orig  x
   ! A ## B = matrix product AB
   ! matrix = m3 # ( m2 # m1)  = m1 m2 m3 ! here was the bug

   m1(:,1) = (/ c1, s1, ze /)
   m1(:,2) = (/-s1, c1, ze /)
   m1(:,3) = (/ ze, ze, un /)

   m2(:,1) = (/ c2, ze, s2 /)
   m2(:,2) = (/ ze, un, ze /)
   m2(:,3) = (/-s2, ze, c2 /)

   m3(:,1) = (/ un, ze, ze /)
   m3(:,2) = (/ ze, c3, s3 /)
   m3(:,3) = (/ ze,-s3, c3 /)

   matrix = matmul(m1,matmul(m2,m3))

end subroutine euler_matrix
